/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
 $Header$
 $Revision$
******************************************************************************
BestFitSmall

Compare two small molecules and find the largest common subgraph.
******************************************************************************/

#include "Graph.h"
#include "GraphStatics.h"

int BestFitSmall (SmallMolecule *sm1, SmallMolecule *sm2,
                  int nmols1, int nmols2, double *hom1, double *hom2)
{
Graph *pgraph1=NULL,*pgraph2=NULL,*pdocked=NULL;
Graph *preduced1=NULL,*preduced2=NULL;
int *nodetype1=NULL,*nodetype2=NULL;
int *node1=NULL,*node2=NULL;

#ifdef DBG
fprintf (stdout,"DBG >> BestFitSmall() - molecules : [%d:%d]\n",nmols1,nmols2);
#endif

/*
 * Make graphs and reduced graphs for
 * the two small molecules
 */

if (MakeGraph(sm1,&pgraph1,&nodetype1) != TRUE)
	{
	ErrTrace ("BestFitSmall - 1", GRAPH_ERR);
	return (FALSE);
	}

if ((preduced1=(Graph *)ReduceGraph(pgraph1))==NULL)
	{
	ErrTrace ("BestFitSmall - 2", GRAPH_ERR);
	return (FALSE);
	}

if (MakeGraph(sm2,&pgraph2,&nodetype2) != TRUE)
	{
	ErrTrace ("BestFitSmall - 3", GRAPH_ERR);
	return (FALSE);
	}

if ((preduced2=(Graph *)ReduceGraph(pgraph2))==NULL)
	{
	ErrTrace ("BestFitSmall - 4", GRAPH_ERR);
	return (FALSE);
	}

/*
 * Generate the docking graph
 */

if ((pdocked=DockingGraph(preduced1,nodetype1,
                      preduced2,nodetype2,
                      &node1,&node2)) != NULL)
	{
/*
 * Now send of the clique finder on 
 * the graph that we have generated.
 */
	if (FindCliques (pdocked, sm1, sm2, nmols1, nmols2, node1, node2) != TRUE)
		{
		ErrTrace ("BestFitSmall - 6", GRAPH_ERR);
		return (FALSE);
		}
	}
else
	fprintf (stdout,"BestFitSmall() WARNING ==> Probably no bonds defined in one of the molecules !\n");

/*
 * Finally free up the graphs and reduced graphs
 */

if (FreeGraph (pgraph1) != TRUE ||
    FreeGraph (pgraph2) != TRUE ||
    FreeGraph (preduced1) != TRUE ||
    FreeGraph (preduced2) != TRUE ||
    FreeGraph (pdocked) != TRUE)
	{
	ErrTrace ("BestFitSmall - 7", FREE_ERR);
	return (FALSE);
	}
#ifdef DBG
fprintf (stdout,"DBG >> BestFitSmall() - Graphs free'd up\n");
#endif
free ((void *)node1);
free ((void *)node2);
free ((void *)nodetype1);
free ((void *)nodetype2);

*hom1=(double)((double)maxcliquesize/(double)(sm1->nbonds))*100.0;
*hom2=(double)((double)maxcliquesize/(double)(sm2->nbonds))*100.0;

#ifdef DBG
fprintf (stdout,"DBG >> BestFitSmall() - done\n");
#endif
return (TRUE);
}
