/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
 $Header$
 $Revision$
*****************************************************************************/

#include "Graph.h"

Graph* DockingGraph(const Graph *preduced1,const int *nodetype1,
		  const Graph *preduced2,const int *nodetype2,
		  int **node1,int **node2)
{
Graph *pgraph;
int nnodes,i,j,k,*pM1in,*pM2in,node1_i,node1_j,node2_i,node2_j,nnodes1,nnodes2;

#ifdef DBG
fprintf (stdout,"DBG >> DockingGraph()\n");
#endif

/* 
 * A node in the docking graph consist of a node from reduced graph 1 and a node
 * from reduced graph 2, each of similar type: 
 */

pgraph=NULL;
if (preduced1!=NULL && preduced2!=NULL)
	{
	nnodes1=preduced1->nnodes;
	nnodes2=preduced2->nnodes;
	/* find the number of matching nodes in the two graphs:*/
	nnodes=0;
	for (i=0;i<nnodes1;i++)
		{
		for (j=0;j<nnodes2;j++)
			{
			if (nodetype1[i]==nodetype2[j])
				{
				nnodes++;
				}
			}
		}

	if (nnodes>0)
		{
		*node1=(int*)calloc((size_t)(nnodes+1),sizeof(int));
		*node2=(int*)calloc((size_t)(nnodes+1),sizeof(int));

		/* store matching nodes in the two graphs:*/
		k=0;
		for (i=0;i<nnodes1;i++)
			{
			for (j=0;j<nnodes2;j++)
				{
				if (nodetype1[i]==nodetype2[j])
					{
					(*node1)[k]=i;
					(*node2)[k]=j;
					k++;
					}
				}
			}

		/* allocate the graph */
		pgraph=AllocGraph(nnodes);

		/* find connections */
		for (i=0;i<nnodes;i++)
			{
			node1_i=(*node1)[i];
			node2_i=(*node2)[i];
			pM1in=preduced1->M+node1_i*nnodes1;
			pM2in=preduced2->M+node2_i*nnodes2;
			for (j=i+1;j<nnodes;j++)
				{
				node1_j=(*node1)[j];
				node2_j=(*node2)[j];
				if (node1_i!=node1_j && node2_i!=node2_j &&
					HasEqualPath(*(pM1in+node1_j),*(pM2in+node2_j)))
					{
					/* here was a connection: */
					pgraph->M[i*nnodes+j]=1;
					pgraph->M[j*nnodes+i]=1;
					}
				}
			}
		}
	else
		{
		*node1=NULL;
		*node2=NULL;
		pgraph=NULL;
		}
	}
else
	{
	*node1=NULL;
	*node2=NULL;
	pgraph=NULL;
	}

#ifdef DBG
fprintf (stdout,"DBG >> DockingGraph() - done\n");
#endif
return pgraph;
}
