/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
 $Header$
 $Revision$
*****************************************************************************/

#include "Graph.h"
#include "GraphStatics.h"


int FindCliques (Graph *pdocked, SmallMolecule *sm1, SmallMolecule *sm2,
                 int nmols1, int nmols2, int *node1, int *node2)
{
int i,j, nn, mm;
int *all;
int maxnbonds,mincliquesize;
double tolerance=80;

#ifdef DBG
fprintf (stdout,"DBG >> FindCliques()\n");
#endif

/*
 * Find the cliques
 */

if (pdocked!=NULL)
	{
        nn=pdocked->nnodes;
	mm = nn+1;
#ifdef DBG
fprintf (stdout,"DBG >> FindCliques() - allocating : %d ints\n",nn);
#endif
	if ((all = (int *)calloc (mm, sizeof(int)))==NULL)
		{
		fprintf (stdout,"Error : Unable to allocate in MakeGraph()\n");
		ErrTrace ("MakeGraph", OUTOFMEM);
		return (FALSE);
		}
/* 
 * set up data for clique finding: 
 */
        maxcliquesize=0;
/*!!!clique_no=0;*/
        for (i=0;i<=mm;i++)
            	{
              	for (j=0;j<=mm;j++)
                 	connected[i][j]=0;
              	compsub[i]=0;
            	}
        for (i=0;i<nn;i++)
            	{
              	for (j=0;j<nn;j++)
              		{
                	if (pdocked->M[i*nn+j])
                		{
                  		connected[i+1][j+1]=1;
                  		connected[j+1][i+1]=1;
                		}
              		}
            	}
#ifdef DBG
fprintf (stdout, "DBG >> FindClicques() - step 1\n");
#endif
        for (i = 0; i < mm; i++)
            	{
              	all[i] = i;
              	connected[i][i]=1;
            	}
        nel = 0;

/* 
 * compute the clique: 
 */
        Extend(all, 0, nn);
#ifdef DBG
fprintf (stdout, "DBG >> FindClique() maxcliquesize : %d\n",maxcliquesize);
#endif
/* 
 * maxclique now contain data from 1..maxcliquesize :
 */

        maxnbonds=sm1->nbonds;
        if (maxnbonds<sm2->nbonds)
              	maxnbonds=sm2->nbonds;
        mincliquesize=(int)(tolerance/100.0*maxnbonds);
        if (maxcliquesize>=mincliquesize)
                {
/* 
 * print clique (i is the number of nodes): 
              	PrintHomology(node1,node2,sm1,sm2,maxclique,
                maxcliquesize,nmols1,nmols2,(maxcliquesize/(double)maxnbonds)*100);
 */
            	}
	free ((void *)all);
	}
else
	{
	fprintf (stdout, "Error : FindCliques() pdocked is NULL\n");
	ErrTrace ("FindCliques", INVALID_VALUE);
	return (FALSE);
	}
#ifdef DBG
fprintf (stdout,"DBG >> FindCliques() - done\n");
#endif
return (TRUE);
}
