
/*
 * Functions:
 */

extern void SetNoPaths(int *ppath_lengths);
  /* deletes all paths in path_lengths. */

extern int HasEqualPath(int path_lengths_a,int path_lengths_b);
  /* returns true/false. */

/* Graph functions: */

extern int Extend(int old[], int ne, int ce);
extern Graph* AllocGraph(int nnodes);
extern int FreeGraph(Graph *pgraph);
extern Graph* AllocGraph(int nnodes);
extern void ErrTrace (char *routine, int ierror);
extern void AllShortestPaths(const Graph *pgraph,Graph *preduced);

extern void PrintPathLengths (int path_length);

extern int GetAtomType (char *atom);

extern Graph* ReduceGraph(const Graph *pgraph);
  /* input: a graph where M[i][j]==1 iff there is an 
            edge between nodes i and j. Otherwise M[i][j]==0.
     output: a reduced graph where M[i][j]==all different path lengths 
            between nodes i and j. 
     on return you will have resonsibility for free'ing the returned graph. */


extern Graph* DockingGraph(const Graph *preduced1,const int *nodetype1,
                    const Graph *preduced2,const int *nodetype2,
                    int **node1,int **node2);
/* input: 
      two reduced graphs: preduced1 and preduced2
      nodetype1: array with the type of each node in graph 1
      nodetype2: array with the type of each node in graph 2
   output: a docking graph representing overlap in the two reduced graphs
      (*node1)[i] is the node in reduced graph 1 which is part of node i in the docking graph. 
      (*node2)[i] is the node in reduced graph 2 which is part of node i in the docking graph. 
   on return you will have resonsibility for free'ing node1, node2 and the returned graph. */

extern void Printint(int path_length);
extern void PrintGraph(const Graph *pgraph);
extern void PrintGraph(const Graph *pgraph);
extern void PrintDockedGraph(const Graph *pgraph,int *node1,int *node2);
  /* to stdout */

extern SmallMolecule* ReadSmallMolecule(FILE *pf,int *perr);
/* reads a molecule from an sdf file.
     If there was not enough memory then the file pointer 
     will be set back to the position it had when the function was called.
     If there was an other error in reading the molecule from the file then
     an attempt will be made to set the file pointer to the start of the next 
     molecule in the file.
   input:
     pf: must be open and the next line must include a molecule name.
   output:
     *perr: NO_ERR if there is no error, 
            MEM_ERR if memory could not be allocated or 
            FILE_ERR if a molecule could not be read from pf.
   return: 
     a pointer to the read molecule.
*/


extern int MakeGraph(SmallMolecule *psmall, Graph **ppgraph, int **ppnodetype);
/* Create a graph and the corresponding node types from 
   a small molecule. The nodes in the graph will be the bonds in 
   the molecule. The nodetypes will be the bond types and the 
   atom types of the to atoms of the bond. */

extern void PrintHomology(int *node1,int *node2,
                SmallMolecule *psmall1,SmallMolecule *psmall2, 
                int *clique,int cliquesize,int nmol1,int nmol2,
                                              double homology);
/*
  Outputs a homology to the file. 
  input: 
     fp1: the location of the SDF print of the homology.
     fp2: connection between homology name and molecule names.
     psmall1: pointer to the first molecule.
     psmall2: pointer to the second molecule.
     node1: array of the nodes in the reduced graph of psmall1
       this is actually an array of bonds in psmall1.
     node2: array of the nodes in the reduced graph of psmall2
       this is actually an array of bonds in psmall2.
     clique: array of nodes in the docking graph for the two 
       reduced graphs.
     cliquesize: the number of nodes in the clique.
     nmol1: identification number of molecule in psmall1.
     nmol2: identification number of molecule in psmall2.

  The homology is printed using the subset of psmall1 which is part
  of the homology.
  Presently psmall2 and node 2 are not used! 
*/

extern int CompareFiles(char *infile1,char *infile2,char *outfile1,
                 char * outfile2,double tolerance);
/*
  Find homologies between molecules in file1 and file2 which are 
  at least tolerance equal. 0<=tolerance<=1.
  Such homologies are printed to outfile1 in SDF format.
  The names of the molecules in the homology is written to outfile2.
  Errors are printed to stderr.
  return: 0 iff no fatal errors were encountered.
*/

extern int FindCliques (Graph *pdocked, SmallMolecule *sm1, SmallMolecule *sm2,
                 int nmols1, int nmols2, int *node1, int *node2);

