/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
 $Header$
 $Revision$
*****************************************************************************/

#include "Graph.h"

int MakeGraph(SmallMolecule *psmall, Graph **ppgraph, int **ppnodetype)
{
Graph *pgraph1=NULL,*pgraph=NULL;
int i,j,k,n,m,natoms,nbonds,ntype,atype_n,atype_m,min_atype,max_atype;

#ifdef DBG
fprintf (stdout,"DBG >> MakeGraph()\n");
#endif

if (psmall!=NULL)
	{
	nbonds=psmall->nbonds;
	natoms=psmall->natoms;

	if ((*ppnodetype=(int *)calloc((nbonds+1),sizeof(int))) == NULL)
		{
		ErrTrace ("MakeGraph",OUTOFMEM);
		return (FALSE);
		}
/* 
 * we construct pgraph1 as a temporary help in
 * building the final ppgraph. pgraph1 has atoms as nodes
 * and bonds as edges. We store which bond that connects to
 * bonded atoms: 
 */
	if ((pgraph1=AllocGraph(natoms)) == NULL)
		{
		ErrTrace ("MakeGraph", OUTOFMEM);
		return (FALSE);
		}

	for (i=0; i<natoms; i++)
		{
		for (j=0; j<natoms; j++)
			(*pgraph1).M[i*natoms+j]=-1;
		}

	for (i=0; i<nbonds; i++)
		{
		n=psmall->bonds[i][0];
		m=psmall->bonds[i][1];
		(*pgraph1).M[n*natoms+m]=i;
		(*pgraph1).M[m*natoms+n]=i;
		}
	for (i=0; i<nbonds; i++)
		{
		n=psmall->bonds[i][0]-1;
		m=psmall->bonds[i][1]-1;
/* 
 * the nodes in the resulting graph will be the edges of pgrap1.
 * we also need a type for each of these edges. Therefore we
 * calculate a node type (ntype) dependent only on the
 * bond type, the two atom types of the end atoms and the number of
 * bonds each end atom participates in: 
 */
		atype_n=GetAtomType (psmall->atomtyp[n]);
		atype_m=GetAtomType (psmall->atomtyp[m]);
		if (atype_n<atype_m)
			{
			min_atype=atype_n;
			max_atype=atype_m;
			}
		else
			{
			min_atype=atype_m;
			max_atype=atype_n;
			}
/*
 * This hashing expects 32 bit integers - shitty !
		ntype=psmall->bondtyp[i]+min_atype+max_atype;
 */
		ntype=psmall->bondtyp[i];
		ntype=ntype<<28;
		ntype=ntype|(min_atype<<14);
		ntype=ntype|max_atype;

/*  
 * store the nodetype: 
 */
		(*ppnodetype)[i]=(int)ntype;
		}
	if ((*ppgraph=AllocGraph(nbonds))==NULL)
		{
		ErrTrace ("MakeGraph", OUTOFMEM);
		return (FALSE);
		}
	pgraph=*ppgraph;
	for (i=0; i<natoms; i++)
		{
		for (j=0; j<natoms; j++)
			{
			for (k=0; k<natoms; k++)
				{
				n=(*pgraph1).M[i*natoms+j];
				/* n is the bond between i and j */
				m=(*pgraph1).M[j*natoms+k];
				/* m is the bond between j and k */

				if (n>=0 && m>=0 && n!=m)
					{
/* 
 * we can say that the two bonds are connected by atom j.
 * therefore we add an edge in the resulting graph: 
 */
					(*pgraph).M[n*nbonds+m]=1;
					(*pgraph).M[m*nbonds+n]=1;
					}
				}
			}
		}
#ifdef DBG
fprintf (stdout,"DBG >> MakeGraph() - Freeing up temporary graph\n");
#endif
	FreeGraph((Graph *)pgraph1);
	}
#ifdef DBG
fprintf (stdout,"DBG >> MakeGraph() - done\n");
#endif
return (TRUE);
}

