/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
 $Header$
 $Revision$
*****************************************************************************/

#include "Graph.h"

void PrintHomology(int *node1,int *node2,
                SmallMolecule *psmall1,SmallMolecule *psmall2,
                int *clique,int cliquesize,int nmol1,int nmol2,
                double homology)
{
int i,j,bond,atom,bondtype;
int *patom,*patom_inv;
int natoms;

#ifdef DBG
fprintf (stdout,"DBG >> PrintHomology()\n");
#endif

patom=(int *)calloc((psmall1->natoms+1),sizeof(int));
patom_inv=(int *)calloc((psmall1->natoms+1),sizeof(int));


/* 
 * count the number of atoms in the homology, and
 * store info in patom and patom_inv:
 */
for (i=1; i<=psmall1->natoms; i++)
	{
	patom_inv[i]=0;
	}
natoms=1;
for (j=1; j<=cliquesize; j++)
	{
	bond=node1[clique[j]-1];
	atom=psmall1->bonds[bond][0]+1;
	if(!patom_inv[atom])
		{
		patom[natoms]=atom;
		patom_inv[atom]=natoms;
		natoms++;
		}
	atom=psmall1->bonds[bond][1]+1;
	if(!patom_inv[atom])
		{
		patom[natoms]=atom;
		patom_inv[atom]=natoms;
		natoms++;
		}
	}

natoms--;

/* write the homology: */
/* fprintf(fp,"HOM_%s_%s\n",psmall1->molname,psmall2->molname); */

/* molname must be at most 12 chars :*/
fprintf(stdout,"H%i_%i : %s %s %g\n",nmol1,nmol2,psmall1->molname,psmall2->molname
						 ,homology);
fprintf(stdout,"H%i_%i\n",nmol1,nmol2);
fprintf(stdout," \n");
/* number of atoms and bonds: */
/*fprintf(stdout,"					 3 99813322D HOM_%s_%s\n",psmall1->molname,psmall2->molname); */
/* molname must be at most 12 chars :*/
fprintf(stdout,"					 3 99813322D H%i_%i\n",nmol1,nmol2);
fprintf(stdout,"%3i%3i				0								 1 V2000\n",natoms,cliquesize);
/* atoms: */
for (i=1; i<=natoms; i++)
	{
	fprintf(stdout,"%10.4f%10.4f%10.4f %-2s	 0	0	0	0	0	0	0	0	0\n",
		psmall1->atoms[patom[i]-1][0],
		psmall1->atoms[patom[i]-1][1],
		psmall1->atoms[patom[i]-1][2],
		psmall1->atomtyp[patom[i]-1]);
	}
	/* bonds: */
for (i=1; i<=cliquesize; i++)
	{
	bond=node1[clique[i]-1];
	bondtype=psmall1->bondtyp[bond];

	fprintf(stdout,"%3i%3i%3i	0	0	0	0\n",
		patom_inv[psmall1->bonds[bond][0]+1],
		patom_inv[psmall1->bonds[bond][1]+1],
		bondtype);
	}
/* end: */
fprintf(stdout,"M	END\n");
fprintf(stdout,"$$$$\n");
fflush(stdout);
fflush(stdout);

free(patom);
patom=NULL;
free(patom_inv);
patom_inv=NULL;
#ifdef DBG
fprintf (stdout,"DBG >> PrintHomology() - done\n");
#endif
}

