/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
* $Header: /usr/people/jan/progs/J/Source/Types/RCS/Simulation.h,v 1.1 1996/03/20 17:44:00 jan Exp jan $
* $Revision: 1.1 $
******************************************************************************/
typedef struct _Simulation
	{
	double  ***v;                  	/* Simulation coordinates - two sets ! */
        double  ***gv;                  /* Simulation group centers - charged atoms only ! for switching */
        double  ***dist;                /* Distance matrix      */
	double  ***gdist;               /* Group distances */
	double  **access;               /* Accessibility - two sets (atom based) */
	double  **dihedral;             /* Dihedrals of current conformation */
	double  *move_angle;            /* Move angle for fb-type simulations*/
        int     set;                    /* Current set */
/* GBorn related data */
	double	*vol_eff;		/* Atomic effective volumes */
	double	*G_self;		/* Atomic self polarization energies */
	double	*radi_brn;		/* Atomic effective Born radii */

        int 	nres;               	/* Number of residues   */
        int 	ngroup;             	/* Number of groups     */

	int  	*torres;                /* Pointer to first torsion in each residue */
	int     ntorres;                /* Number of residues with a torsion */

	int	**group;		/* Group pointers */
        int	*ingroup;		/* Number of atoms in */

	int  	*nonpolar;		/* Pointers to nonpolar groups */
        int	Nnonpolar;              /* Number of these groups */
        int	NAnonpolar;             /* Number of atoms */

        int     *polar;			/* Polar groups */
        int	Npolar;			/* Number of polar groups*/
        int	NApolar;		/* Number of polar atoms */

	int	*charged;		/* Charged groups */
	int	Ncharged;		/* Number of groups */
	int	NAcharged;		/* Number of atoms */
/* Coulomb energy */
	double  *Ecoul;                	/* Coulomb energy (group based) */
        double   TEcoul;             	/* Total Culoumb energy */
        double   TEborn;                /* The GB solvation (polarization energy according to Rui */
/* Elocal stuff */
	int    **ElocPointer;        	/* Elocal pointers the first group is the one we are looking at */
	int     *NElocPointer;       	/* Number of pointers per group */
        double  *ElocScale;          	/* Eloc scalingfactor */
        double  *Eloc;               	/* Atomic contributions */
        double  *Ehbcorr;              	/* Correction to put onto HB-corr */
        double   TEloc;              	/* Total eloc */
/* Ehb - hydrogen bonding */
	int     *Ehbscale;              /* Ehb group type scaling factor index (group based) */
	double  *Ehb;                	/* Ehb */
        double   TEhb;               	/* hydrogen bonding part */
/* Eother - sidechain electrostatics */
	int     *Eotscale;              /* Eother group type scaling factor index (group based) */
	double  *Eother;             	/* Eother */
        double   TEother;            	/* other charges and dipoles */
/* Enonlocal - hydrogens according to avbelj */
	double  *EnonlocScale;       	/* Enonlocal scale factors (group based) */
        double  *Enonloc;              	/* Atomic contributions */
        double   TEnonloc;             	/* Total non eloc */
/* Hydrophobic energy */
 	double  *ShRup;              	/* Shrake & Rupley areas (atom based) */
        double  *Ephobe;             	/* Ephobes (group based) */
 	double   TEphobe;            	/* Total Ephobe */
/* Solvation free energy */
        double  *Esolv;              	/* Esolvation  (group based) */
        double   TEsolv;             	/* Total Esolv */
/* Burial energy */
        double  *Ebur;               	/* Burial energy */
        double   TEbur;              	/* Total Burial energy */
/* Chi potential */
        double  *Echi;               	/* Chi preference term */
	int     *ChiPointers;           /* Pointer to last atom in each residues torsions */
        double   TEchi;              	/* Total chi energy */
/* Lennard-Jonesium */
	double  *Elj;                   /* Lennard Jones energy */
        double   TElj;                  /* Total Lennard Jones energy */
/* Contact potential */
	double	 TEcontact;		/* Total contact energy */
/* Tethering energy - forcing ! */
	double	 TEtether;		/* Total thether energy */
/* Clash check - treated as all other energies */
	int  	 Eclash;                /* Clash flag */
/* Pair potential energies */
	double  *Epair;			/* Atomic pair potential energy */
	double   TEpair;		/* Total pair potential energy */
	int     *PairPointer;		/* Pointer to atom type in parameter list */
/* Elocal according to Jan */
        int    **EJlocPointer;          /* Elocal pointers the first group is the one we are looking at */
        int     *NJElocPointer;         /* Number of pointers per group */
        double  *EJlocScale;            /* Eloc scalingfactor */
        double  *EJlocScale2;           /* Eloc scalingfactor */
        double  *EJloc;                 /* Atomic contributions */
        double   TEJloc;                /* Total eloc */
/* Hydrogen bonding according to Jan */
        double  *EJnonlocScale;         /* Enonlocal scale factors (group based) */
        double  *EJnonlocScale2;        /* Enonlocal scale factors (group based) */
        double  *EJnonloc;              /* Atomic contributions */
        double   TEJnonloc;             /* Total non eloc */
/* PMF Elocal and Enonlocal pairtype */
	int 	*PMFpairtyp;		/* The PMF pair type - index to a JELocalNonLocal2  (residue pair based) */
/* All of the above */
	double   Etotal;                /* The total energy */
	double   Esav;			/* Previous total energy */
	double   dE;                    /* The energy difference to last calculation */

	double	temperature;		/* Current temperature of system */
	double	rmsd;			/* Rmsd to reference structure */
	double	Apolar;			/* Total polar area */
	double	Anonpolar;		/* do nonpolar */
	double  Acharged;		/* do charged */
	int	Accept;			/* was the current structure accepted ? */
	double	CPUTime;		/* Cpu time since last call */

	int	*fixlist;		/* List of fixed torsions */
	int	*tetherlist;		/* List of thethered torsions */
	int 	**constlist;		/* List of constraints */
        double  *constdist;     	/* Constraint distance */
	double  *constfc;		/* Constraint force constant */
	int 	nconst;			/* The number of constraints */
	double  TEguide;		/* Total guide energy */

	void 	*stats;             	/* Pointer to statistics for this molecule during simulation */
	void 	*derivatives;		/* structure for derivatives */
/* Docking stuff */
	double  **surface;		/* Surface dots */
	int     *surface_atom;		/* Which atoms does a dot belong to ? */
	int 	nsurf;			/* Number of surface points */
/* Spheres */
	double	**spheres;		/* A set of surface spheres  - center and radius and a fifth parameter*/
	int 	**sphere_atom;		/* atoms touching spheres */
	int 	nsphere;		/* Number of spheres */
/* Sphere map */
        int 	**sphere_map;		/* For each atom map all the possible spheres */
	int 	*nmap_sphere;           /* How many spheres are mapped to this atom */
/* Receptor map */
	int	**rec_map;		/* Receptor atoms - on residue basis */
  	int 	*nmap_rec;		/* Number of atoms in residue */
	int 	nrec_res;		/* Number of receptor residues */
/* NOE calculation pointers */
 	int	**NoePt;		/* pointers to Ca, N, C, HN, Cb in each residue */
	} Simulation;
