/*****************************************************************************

                'J' - Yet Another Molecular Mechanics Program

                           (c) 1995 Jan T Pedersen


This program is not in the public domain and may not be used or distributed
by anyone without the written permission of the author. The functionality
of the whole or parts of the program is not guaranteed.

******************************************************************************
* $Header: /usr/people/jan/progs/J/Source/Types/RCS/Variables.h,v 1.1 1996/03/20 17:44:00 jan Exp jan $
* $Revision: 1.1 $
******************************************************************************

 Variables.h

 Datastructure to hold simulation variables and which can be handled by
 a variable manager.

******************************************************************************/
#define NVAR   		47
#define NMOV_TYPES	13

#define J_SIM_VAR	0
#define J_ENE_VAR	1
#define J_ALL_VAR	2
#define J_CAV_VAR	3

typedef struct _Variables
	{
	char **varname;	  		/* Variable names */
	char **vardesc;			/* Variable description */
	int *varclass;          	/* Variable class */
	int *vartype;  	        	/* Variable types */
        int nvar;   			/* soft variable counter */

	double Overlap;			/*     Acceptable overlap distance */
 	double DistanceCutoff;        	/*  1  Cutoff for group-group interactions                   */
 	double DielectricProtein;   	/*     Protein dielectric constant                           */
 	double DielectricSolvent;   	/*     Solvent dielectric constant                           */
        double IonicStrength; 	   	/*     Ionic strength (Molar)                                */
        double ExclusionRadius;		/*  5  Ionic exclusion radius                                */
	int    Monitor;			/*     Steps between coordinate sampling                     */
	int    RandomSeed; 		/*     Random number seed                                    */
	double Temperature; 		/*     Absolute temperature of run (Kelvin)                  */
	double TemperatureGradient; 	/*     Van der Waals energy truncation                       */
	double VdWMax; 			/* 10  Total number of steps in run                          */
	int    NSteps; 			/*     Tolerance for max pertubation sampling (Angst.)       */
	double BurialCutoff; 		/*     Cutoff for atom burrial (Sqr Angstroem)               */
	double MolecularRadius; 	/*     Effective radius of molecule (Angstroem)              */
	double ChargeBurial; 		/*     Minimum burrial distance for charge (Angstroem)       */
	double AnnealingCutoff; 	/* 15  Energy cut-off for annealing                          */
	int    AnnealingSteps; 		/*     Number of steps/temp step in annealing                */
	double AcceptanceRate; 		/*     Accptance rate nacc/nrej cut-off for annealing        */
	double FinalTemperature; 	/*     Final temperature of annealing run                    */
	double MoveAngle; 		/*     Maximum relative move angle for RELMOVE search        */
	int    SpawnRate; 		/* 20  Rate of spawning for constant spawning annealing      */
	int    MinimumFrame; 		/*     Minimum sequence frame for context burrial            */
	int    MaximumFrame; 		/*     Maximum sequence frame for context burrial            */
	double AccessibilityCutoff; 	/*     Number of generations in GA simulation                */
	int    Generations; 		/*     Number of angle picks in each cross-over              */
	double CarryOverFraction; 	/* 25  Percentage of conformations to leak in each GA cyc    */
	int    SDRecalculation; 	/*     Re-calculation frequency of SDeloc in GA run (-1 e    */
 	double WaterRadius;		/*     Radius of water probe				     */
        double SliceThickness;          /*     Sectioning thickness for surface routine		     */
        double TetherConstant;          /*     Force constant for tethering                          */
        int    NPick;		        /* 30  Number of GA joint picks in cross-over                */
        double LJScale;                 /*     Lennard Jones scaling factor             	     */
        double FBLambda;                /*     Force Bias lambda for evaluation of probability       */
        double MinSphere;		/*     Minimum sphere radius                                 */
        double MaxSphere;               /*     Maximum sphere radius                                 */
        int    ResidueSeparation;       /* 35  Residue separation for accepting spheres              */
        double DotOverlapCutoff;        /*     Overlap for eliminating dots on the surface           */
        int    Tesselation;             /*     Icosahedron tesselation for generation of surface     */
        double Esphere;                 /*     Energy cutoff for potential assignment                */
        double TargetDeviation;         /*     Deviation on translation - random pertubation         */
        int    SubMCStep;               /* 40  Substrate MC Steps / Dock step                        */
        int    RecMCStep;               /*     Receptor MC Steps / Dock step                         */
        int    DockStep;                /*     Docking steps between MC runs                         */
	int    TransformationAttempts;  /*     Attempts at finding orientation for seed              */
	int    DockMinStep;             /*     Rigid body minimisation steps for each docking step   */
	int    GAMcSteps;               /* 45  Steps of Monte Carlo simulation between cross-over    */
	double HPScale;                 /*     Scaling of hydrophobic energy                         */


	double MoveFractions[NMOV_TYPES];	/* Move Mix fractions */
	int    MoveTypes[NMOV_TYPES];		/* Move Type mix */
 	char   MoveNames[NMOV_TYPES][MAXNAME];  /* Move type names */
 	char   MoveDesc[NMOV_TYPES][MAXWORD];  /* Move type names */
	} Variables;
